package com.rf.check.user.rest;

import com.alibaba.fastjson.JSONObject;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.google.gson.Gson;
import com.rf.check.base.rest.BaseController;
import com.rf.check.filter.JwtIgnore;
import com.rf.check.picture.dao.model.PictureEntity;
import com.rf.check.security.SafetyProcess;
import com.rf.check.user.dao.model.UserCheckEntity;
import com.rf.check.user.dao.model.UserEntity;
import com.rf.check.user.dao.model.UserPictureEntity;
import com.rf.check.user.service.UserCheckService;
import com.rf.check.user.service.UserPictureService;
import com.rf.check.user.service.UserService;
import com.rf.check.utils.Constant;
import com.rf.check.utils.DateUtil;
import com.rf.check.utils.JWTUtil;
import com.rf.check.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

/**
 * @Description: 用户注册登录等相关接口
 * @Author: zsy
 * @Date: 2024/12/4
 */
@RestController
@RequestMapping("/user")
@Api(tags = "用户注册登录等相关接口")
@Slf4j
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    @Autowired
    private UserCheckService userCheckService;

    @GetMapping("/getToken")
    @ApiOperation(value = "获取token", notes = "")
    @JwtIgnore
    public Result getToken() {
        UserEntity userEntity = this.userService.findByOpenId("o_QoB7K8Q-tOyic9G3nvxDwvIKQ4");
        return success(JWTUtil.getTokenByUserInfo(userEntity));
    }

    @GetMapping("/code2openid/{code}")
    @ApiOperation(value = "code 换 opernid",notes = "code：临时code，获取openid")
    @SafetyProcess
    public Result<JSONObject> code2openid(@PathVariable String code) throws Exception {
        String openId;
        HttpResponse response = getToken(code);
        int statusCode = response.getStatusLine().getStatusCode();//响应状态码
        String body = EntityUtils.toString(response.getEntity());
        Gson gson = new Gson();
        HashMap resultBody = gson.fromJson(body, HashMap.class);
        if(statusCode == 200 || statusCode == 204){
            openId = (String) resultBody.get("openid");
            if (openId == null){
                log.error("错误码：{}",resultBody.get("errcode"));
                log.error("错误消息：{}",resultBody.get("errmsg"));
                return fail(String.valueOf(resultBody.get("errmsg")));
            }
        }else{
            log.error("获取openid失败：错误码 "+resultBody.get("errcode")+"  "+resultBody.get("errmsg"));
            log.error("错误码：{}",resultBody.get("errcode"));
            log.error("错误消息：{}",resultBody.get("errmsg"));
            return fail();
        }
        JSONObject resultJson = new JSONObject();
        //静默注册登录
        UserEntity userEntity = this.userService.findByOpenId(openId);
        if(userEntity != null){
            resultJson.put("improve",false);
        }else{
            //注册
            userEntity = new UserEntity();
            userEntity.setCreateTime(DateUtil.getNowTime());
            userEntity.setUpdateTime(DateUtil.getNowTime());
            userEntity.setUserName("微信用户");
            userEntity.setOpenId(openId);
            userEntity.setPhone("-");
            userEntity.setGender("0");
            userEntity.setCheckNum(0);
            Random rand = new Random();
            int number = rand.nextInt(3) + 1;
            if (userEntity.getGender().equals("1")){
                userEntity.setAvatar("./userAvatar/man"+number+".png");
            }else {
                userEntity.setAvatar("./userAvatar/woman"+number+".png");
            }
            userEntity = this.userService.save(userEntity);
            resultJson.put("improve",true);
        }

        resultJson.put("token", JWTUtil.getTokenByUserInfo(userEntity));
        resultJson.put("user",userEntity);
        resultJson.put("openId",openId);
        log.info("响应消息："+resultJson.toJSONString());
        return success(resultJson);
    }

    public HttpResponse getToken(String code) throws IOException {
        String url = Constant.ASPI_URL +
                "?appid=" + Constant.ASPI_APPID +
                "&secret=" + Constant.ASPI_SECRET +
                "&js_code=" + code +
                "&grant_type=authorization_code";
        log.info("code2openid URL ："+url);
        HttpClient httpClient = HttpClients.createDefault();
        HttpGet httpGet = new HttpGet(url);
        httpGet.setHeader("Accept", "application/json");
        return httpClient.execute(httpGet);
    }

    /**
     * 修改用户信息
     * @param
     * @return
     */
    @ApiOperation(value = "修改用户信息接口",notes = "id:用户id,userName:昵称,phone:手机号,gender:性别(0-男；1-女),avatar:头像图片路径,intro:用户简介")
    @PostMapping("/updateUser")
    public Result updateAvatar(@RequestBody String json){
            UserEntity userEntity = JSONObject.parseObject(json, UserEntity.class);
            UserEntity user = this.userService.findById(userEntity.getId());
            if (user == null) {
                return fail(null, "用户不存在");
            }
            userEntity.setUpdateTime(DateUtil.getNowTime());
            userEntity.setCheckNum(user.getCheckNum());
            userEntity.setOpenId(user.getOpenId());
            this.userService.save(userEntity);
            return success(userEntity);
    }

    /**
     * 上传图片
     * @param
     * @return
     */
    @ApiOperation(value = "上传图片",notes = "avatar:图片文件,userId:用户id,type:图片类型(1-用户头像；2-用户生成图片)")
    @PostMapping("/uploadPictures")
    public Result uploadPictures(String userId, String type, MultipartFile avatar){
        if (avatar == null) {
            return fail("头像为空");
        } else {
            UserEntity userEntity = userService.findById(userId);
            System.out.println("11:"+userEntity);
            if (userEntity == null) {
                return fail(null, "用户不存在");
            }
            String fileName = "";
            String FILEDIR = "";
            if (type.equals("1")) {
                FILEDIR = "./userAvatar/";
            }else {
                String date = DateUtil.getDateStr(new Date(), DateUtil.ACCURACY_PATTERN_MONTH);
                FILEDIR = "./userPictures/" + date + "/";
            }
            String avatarUrl = "";
            if (!avatar.isEmpty()) {
                fileName = userEntity.getOpenId() + "-=-" + System.currentTimeMillis() + "-=-" + avatar.getOriginalFilename();
                try {
                    File temp = new File(FILEDIR);
                    if (!temp.exists()) {
                        temp.mkdirs();
                    }
                    File fileLocal = new File(FILEDIR, fileName);
                    FileUtils.copyInputStreamToFile(avatar.getInputStream(), fileLocal);
                    avatarUrl = FILEDIR + fileName;
                    return success(avatarUrl);
                } catch (Exception e) {
                    e.printStackTrace();
                    return fail("文件上传失败");
                }
            } else {
                return fail("图片为空");
            }
        }
    }

    @GetMapping(value = "/show",produces = MediaType.IMAGE_JPEG_VALUE )
    @ResponseBody
    @ApiOperation(value = "图片展示",notes = "filePath：相对路径")
    public byte[] showPhoto(String filePath) throws Exception {
        try{
            File file = new File(filePath);
            if(file.exists()){
                FileInputStream inputStream = new FileInputStream(file);
                byte [] bytes = new byte[inputStream.available()];
                inputStream.read(bytes,0,inputStream.available());
                return bytes;
            }
            return null;
        }catch(Exception e){
            throw new Exception("未找到图片");
        }
    }

    @PostMapping("/saveCheck")
    @ApiOperation(value = "保存打卡记录", notes = "")
    public Result saveCheck(HttpServletRequest request) throws ParseException {
        //if (isEffectiveDate()){
            String token = request.getHeader("Authorization");
            String[] tokens = token.split(" ");
            if (tokens.length != 2) {
                token = tokens[0];
            }else {
                token = tokens[1];
            }
            JWTUtil.verify(token);
            DecodedJWT verify = JWTUtil.verify(token);
            String userId = verify.getClaim("userId").asString();
            String nowDate = DateUtil.getNowTime("yyyy-MM-dd");
            UserCheckEntity userCheckEntity = this.userCheckService.findByUserIdAndCreateTime(userId,nowDate);
            if (userCheckEntity == null) {
                userCheckEntity = new UserCheckEntity();
                userCheckEntity.setUserId(userId);
                userCheckEntity.setCreateTime(DateUtil.getNowTime());
                userCheckEntity.setUpdateTime(DateUtil.getNowTime());
                userCheckEntity = this.userCheckService.save(userCheckEntity);
                //增加累计打卡次数
                this.userService.checkNumAdd(userId);
                return success(userCheckEntity);
            }else {
                return fail(null, "今日已打卡");
            }
        /*}
        return fail(null, "不在打卡时间范围内");*/
    }

    /**
     * 判断当前时间是否在[startTime, endTime]区间，注意时间格式要一致
     *
     * @return
     */
    public static boolean isEffectiveDate() throws ParseException {
        String format = "HH:mm:ss";
        Date nowTime = new SimpleDateFormat(format).parse(DateUtil.getDateStr(new Date(),DateUtil.DEFAULT_TIME_PATTERN));
        Date startTime = new SimpleDateFormat(format).parse("04:00:00");
        Date endTime = new SimpleDateFormat(format).parse("12:00:00");
        if (nowTime.getTime() == startTime.getTime()
                || nowTime.getTime() == endTime.getTime()) {
            return true;
        }
        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);
        Calendar begin = Calendar.getInstance();
        begin.setTime(startTime);
        Calendar end = Calendar.getInstance();
        end.setTime(endTime);
        if (date.after(begin) && date.before(end)) {
            return true;
        } else {
            return false;
        }
    }

    @GetMapping("/checkRecord")
    @ApiOperation(value = "用户打卡记录列表查询", notes = "参数包括：beginNum:页码，pageSize：每页条数")
    public Result checkRecord(int beginNum, int pageSize, HttpServletRequest request) {
        String token = request.getHeader("Authorization");
        String[] tokens = token.split(" ");
        if (tokens.length != 2) {
            token = tokens[0];
        }else {
            token = tokens[1];
        }
        JWTUtil.verify(token);
        DecodedJWT verify = JWTUtil.verify(token);
        String userId = verify.getClaim("userId").asString();
        Page<UserCheckEntity> userCheckEntities = this.userCheckService.checkRecord(userId, beginNum, pageSize);
        return success(userCheckEntities);
    }

    @GetMapping("/todayRanking")
    @ApiOperation(value = "打卡记录今日排行列表查询", notes = "参数包括：beginNum:页码，pageSize：每页条数")
    public Result todayRanking(int beginNum, int pageSize) {
        String nowDate = DateUtil.getNowTime("yyyy-MM-dd");
        Page<UserCheckEntity> userCheckEntities = this.userCheckService.todayRanking(nowDate, beginNum, pageSize);
        UserEntity userEntity;
        for (int i = 0; i < userCheckEntities.getContent().size(); i++) {
            userEntity = this.userService.findById(userCheckEntities.getContent().get(i).getUserId());
            userCheckEntities.getContent().get(i).setAvatar(userEntity.getAvatar());
            userCheckEntities.getContent().get(i).setUserName(userEntity.getUserName());
        }
        return success(userCheckEntities);
    }

    @GetMapping("/cumulativeRanking")
    @ApiOperation(value = "打卡记录累计排行列表查询", notes = "参数包括：beginNum:页码，pageSize：每页条数")
    public Result cumulativeRanking(int beginNum, int pageSize) {
        Page<UserEntity> userEntities = this.userService.cumulativeRanking(beginNum, pageSize);
        return success(userEntities);
    }

}
